@@ -110,7 +110,7 @@ sub _expand_error {
     while ( @_ ) {
         my $var = shift;
         my $val = shift;
-        $str =~ s/\$\{$var\}/$val/g;
+        $str =~ s/\$\{$var\}/$val/g if defined $str;
     }
 
     $self->{_errtext} = $str;
@@ -239,6 +239,7 @@ for a specific error.
     'elem-empty-but-closed'    => ['<${tag}> is not a container -- </${tag}> is not allowed', STRUCTURE],
     'elem-img-alt-missing'     => ['<img src="${src}"> does not have ALT text defined', HELPER],
     'elem-img-sizes-missing'   => ['<img src="${src}"> tag has no HEIGHT and WIDTH attributes', HELPER],
+    'elem-input-alt-missing'   => ['<input name="${name}" type="image"> does not have non-blank ALT text defined', HELPER],
     'elem-nonrepeatable'       => ['<${tag}> is not repeatable, but already appeared at ${where}', STRUCTURE],
     'elem-unclosed'            => ['<${tag}> at ${where} is never closed', STRUCTURE],
     'elem-unknown'             => ['Unknown element <${tag}>', STRUCTURE],
@@ -269,45 +270,43 @@ Directive values can only be "on", "off", "yes", "no", "true", "false", "0" and
 
 =head2 elem-unknown
 
-Unknown element E<lt>TAGE<gt>
-
-HTML::Lint doesn't know what a TAG tag is.  These are pulled from HTML::Entities
+HTML::Lint doesn't know recognize the tag.
 
 =head2 elem-unopened
 
-E<lt>/TAGE<gt> with no opening E<lt>TAGE<gt>
+C<< </tag> >> with no opening C<< <tag> >>.
 
 =head2 elem-unclosed
 
-E<lt>TAGE<gt> at WHERE is never closed
+C<< <tag> >> at WHERE is never closed.
 
 =head2 elem-empty-but-closed
 
-E<lt>TAGE<gt> is not a container -- E<lt>/TAGE<gt> is not allowed
+C<< <tag> >> is not a container -- C<< </tag> >> is not allowed.
 
 =head2 elem-img-alt-missing
 
-E<lt>img src="FILENAME.PNG"E<gt> does not have ALT text defined
+C<< <img src="FILENAME.PNG"> >> does not have ALT text defined.
 
 =head2 elem-img-sizes-missing
 
-E<lt>img src="FILENAME.PNG"E<gt> tag has no HEIGHT and WIDTH attributes
+C<< <img src="FILENAME.PNG"> >> tag has no HEIGHT and WIDTH attributes.
 
 =head2 elem-nonrepeatable
 
-E<lt>TAGE<gt> is not repeatable, but already appeared at WHERE
+C<< <tag> >> is not repeatable, but already appeared at WHERE.
 
 =head2 doc-tag-required
 
-E<lt>TAGE<gt> tag is required
+C<< <tag> >> tag is required.
 
 =head2 attr-repeated
 
-ATTR attribute in E<lt>TAGE<gt> is repeated
+ATTR attribute in C<< <tag> >> is repeated.
 
 =head2 attr-unknown
 
-Unknown attribute "ATTR" for tag E<lt>TAGE<gt>
+Unknown attribute "ATTR" for tag C<< <tag> >>.
 
 =head2 text-invalid-entity
 
@@ -327,13 +326,16 @@ Character "CHAR" should be written as ENTITY
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2005-2012 Andy Lester.
+Copyright 2005-2016 Andy Lester.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Artistic License v2.0.
 
 http://www.opensource.org/licenses/Artistic-2.0
 
+Please note that these modules are not products of or supported by the
+employers of the various contributors to the code.
+
 =head1 AUTHOR
 
 Andy Lester, C<andy at petdance.com>
